/** -------------------------------------------------------------------------
Copyright (c) 2012 Chuck Broadwell W5UXH

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

**/

#include "mbed.h"
#include "w5uxhMbedKeyer.h"
#include "queues.h"
#include "TextLCD.h"

extern Fifo *cwBuf;
extern Serial pc;
extern TextLCD lcd;

extern unsigned char focus;
extern unsigned char pdlFocusChar;
extern unsigned char kbdFocusChar;
extern unsigned char wtFocusChar;

extern unsigned char lcdDisplayState;
extern bool rigEna;
extern bool monEna;

/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void dsplyProsign(char asciiChar)
{
  char c0, c1;
  bool needSpace= FALSE;

  // We need to handle the LCD and console displays also
  cwBuf->write(asciiChar);
  switch (asciiChar)
    {
    case AS: c0='A'; c1='S'; needSpace=TRUE; break;
    case VE: c0='V'; c1='E'; needSpace=TRUE; break;
    case AR: c0='A'; c1='R'; needSpace=TRUE; break;
    case SK: c0='S'; c1='K'; needSpace=TRUE; break;
    case AA: c0='A'; c1='A'; needSpace=TRUE; break;
    case BT: c0='B'; c1='T'; needSpace=TRUE;  break;
    default: c0='?'; c1='?'; break;
    }
  if (needSpace) cwBuf->write(' ');

  pc.printf(" %c%c ", c0, c1);
  if (asciiChar == BT)
    {
      pc.printf("%\r");
    }

  displayLcdTextRow(2,' ');
  displayLcdTextRow(2,c0);
  displayLcdTextRow(2,c1);
  displayLcdTextRow(2,' ');
}

/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void up5(char asciiChar)
{
  switch(focus)
    {
    case KBD:
      {
    deltaSpeedKbd(5); break;
      }
    case PDL:
      {
    deltaSpeedPdl(5); break;
      }
    case WEIGHT:
      {
    deltaWeight(5); break;
      }
    }

  displaySpeedsSerial();
}

/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void down1(char asciiChar)
{
  switch(focus)
    {
    case KBD:
      {
    deltaSpeedKbd(-1); break;
      }
    case PDL:
      {
    deltaSpeedPdl(-1); break;
      }
    case WEIGHT:
      {
    deltaWeight(-1); break;
      }
    }

  displaySpeedsSerial();
}


/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void down5(char asciiChar)
{
  switch(focus)
    {
    case KBD:
      {
    deltaSpeedKbd(-5); break;
      }
    case PDL:
      {
    deltaSpeedPdl(-5); break;
      }
    case WEIGHT:
      {
    deltaWeight(-5); break;
      }
    }

  displaySpeedsSerial();
}

/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void up1(char asciiChar)
{
  switch(focus)
    {
    case KBD:
      {
    deltaSpeedKbd(1); break;
      }
    case PDL:
      {
    deltaSpeedPdl(1); break;
      }
    case WEIGHT:
      {
    deltaWeight(1); break;
      }
    }

  displaySpeedsSerial();
}


/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void resetCwBuf(char asciiChar)
{
  cwBuf->reset();
}

extern unsigned char weight;
extern unsigned char kbdWPM;
extern unsigned char pdlWPM;
unsigned char kbdWPMCopy;
unsigned char pdlWPMCopy;
extern unsigned char toggleSameSpeedModeState;

/** -------------------------------------------------------------------------
@verbatim
Toggle betweent normal speed settings and the "same speed" mode;  in this mode,
the current paddle speed is copied to the kbd speed after the current kbd speed
is saved for later restoration.

Changes to paddle speed in this mode also apply to the alternate kbd speed while 
in this mode.

@endverbatim
**/
void toggleSameSpeedMode(char asciiChar)
{
  switch(toggleSameSpeedModeState)
    {
    case IDLE:
      {
    toggleSameSpeedModeState = ACTIVE;
    kbdWPMCopy = kbdWPM;
    kbdWPM = pdlWPM;
    pdlWPMCopy = pdlWPM;

    calcTimeDurationsKbd(kbdWPM, weight);
    pdlFocusChar = '=';
    kbdFocusChar = '=';
    displaySpeedsLCD();
    displaySpeedsSerial();


    break;
      }
    case ACTIVE:
      {
    toggleSameSpeedModeState = IDLE;
    kbdWPM = kbdWPMCopy;
    calcTimeDurationsKbd(kbdWPM, weight);

    pdlWPM = pdlWPMCopy;
    calcTimeDurationsPdl(pdlWPM, weight);

    setFocus(0xB6);
    displaySpeedsLCD();
    displaySpeedsSerial();
    
    break;
      }
    }
}

void swapSpeeds(char asciiChar)
{
  unsigned char temp;
  temp=kbdWPM;
  kbdWPM=pdlWPM;
  pdlWPM=temp;

  calcTimeDurationsKbd(kbdWPM, weight);
  calcTimeDurationsPdl(pdlWPM, weight);

  displaySpeedsLCD();
  displaySpeedsSerial();
}

void setFocus(char asciiChar)
{
  switch(asciiChar)
    {
    case 0xB5:
      {
    // PDL
    focus = PDL;
    pdlFocusChar = '=';
    kbdFocusChar = ':';
    wtFocusChar  = ':';

    displaySpeedsLCD();
    displaySpeedsSerial();
    break;
      }

    case 0xB6:
      {
    // KBD
    focus = KBD;
    pdlFocusChar = ':';
    kbdFocusChar = '=';
    wtFocusChar  = ':';

    displaySpeedsLCD();
    displaySpeedsSerial();
    break;
      }

    case 0xB7:
      {
    // WT
    focus = WEIGHT;
    pdlFocusChar = ':';
    kbdFocusChar = ':';
    wtFocusChar  = '=';
 
    displaySpeedsLCD();
    displaySpeedsSerial();
    break;
      }
    }
}

void announcePdlSpeed(char asciiChar)
{
  char tempString[6];

  calcTimeDurationsKbd(SPEED_FOR_ANNOUNCE, weight);
  sprintf(tempString, "%02d", pdlWPM);

  cwBuf->write('P');
  cwBuf->write('D');
  cwBuf->write('L');
  cwBuf->write(' ');
  cwBuf->write(tempString[0]);
  cwBuf->write(tempString[1]);
  cwBuf->write(EOM_RESTORE_KBD_SPD);  // end of announce message
}

void announceKbdSpeed(char asciiChar)
{
  char tempString[6];

  calcTimeDurationsKbd(SPEED_FOR_ANNOUNCE, weight);
  sprintf(tempString, "%3d", kbdWPM);

  cwBuf->write('K');
  cwBuf->write('B');
  cwBuf->write('D');
  cwBuf->write(' ');
  cwBuf->write(tempString[0]);
  cwBuf->write(tempString[1]);
  cwBuf->write(tempString[2]);
  cwBuf->write(EOM_RESTORE_KBD_SPD);  // end of announce message
}

void announceWeight(char asciiChar)
{
  char tempString[6];

  calcTimeDurationsKbd(SPEED_FOR_ANNOUNCE, weight);
  sprintf(tempString, "%2d", weight);

  cwBuf->write('W');
  cwBuf->write('T');
  cwBuf->write(' ');
  cwBuf->write(tempString[0]);
  cwBuf->write(tempString[1]);
  cwBuf->write(EOM_RESTORE_KBD_SPD);  // end of announce message
}

bool tuneFlag = FALSE;
bool tuneState = FALSE;
void toggleTuneState(void)
{
  if(tuneState)
    {
      keyUp();
      tuneState = OFF;
    }
  else
    {
      keyDown();
      tuneState = ON;
    }
}




void toggleRigState(void)
{
  if(rigEna)
    {
      keyUp();
      rigEna = FALSE;
      displayLcdStringRow(3, "Key Enable OFF      ");
      pc.printf("\rKey Enable OFF\r");
    }
  else
    {
      rigEna = TRUE;
      displayLcdStringRow(3, "Key Enable ON       ");
      pc.printf("\rKey Enable ON\r");
    }
}


void toggleMonState(void)
{
  if(monEna)
    {
      keyUp();
      monEna = FALSE;
      displayLcdStringRow(3, "Sidetone OFF        ");
      pc.printf("\rSidetone OFF\r");
    }
  else
    {
      monEna = TRUE;
      displayLcdStringRow(3, "Sidetone ON         ");
      pc.printf("\rSidetone ON\r");
    }
}

extern unsigned char curtisType;
void setTypeA(void)
{
  curtisType = TYPE_A;
  displayLcdStringRow(3, "Curtis Type_A       ");
  pc.printf("\rCurtis TYPE_A\r");
}

void setTypeB(void)
{
  curtisType = TYPE_B;
  displayLcdStringRow(3, "Curtis Type_B       ");
  pc.printf("\rCurtis TYPE_B\r");
}


void funckey(char asciiChar)
{
  switch(asciiChar)
    {
    case 0x94: pc.printf("F5\r");  break;
    case 0x95: setTypeA();         break;
    case 0x96: setTypeB();         break;
    case 0x97: toggleMonState();   break;
    case 0x98: toggleRigState();   break;
    case 0x99: pc.printf("F10\r"); break;
    case 0x9A: pc.printf("F11\r"); break;
    case 0x9B: toggleTuneState();  break;
    default: pc.printf("Unknown key in funckey() code = 0x%02X\r"); break;
    }
}

void funckeyShift(char asciiChar)
{
  switch(asciiChar)
    {
    case 0xC1: pc.printf("F1 shift\r"); break;
    case 0xC2: pc.printf("F2 shift\r"); break;
    case 0xC3: pc.printf("F3 shift\r"); break;
    case 0xC4: pc.printf("F4 shift\r"); break;
    case 0xD4: pc.printf("F5 shift\r"); break;
    case 0xD5: pc.printf("F6 shift\r"); break;
    case 0xD6: pc.printf("F7 shift\r"); break;
    case 0xD7: pc.printf("F8 shift\r"); break;
    case 0xD8: pc.printf("F9 shift\r"); break;
    case 0xD9: pc.printf("F10 shift\r"); break;
    case 0xDA: pc.printf("F11 shift\r"); break;
    case 0xDB: pc.printf("F12 shift\r"); break;
    default: pc.printf("Unknown key in funckeyShift() code = 0x%02X\r"); break;
    }
}

extern int line2Column;
extern int line2BufIndex;
extern int line3Column;
extern int line3BufIndex;

extern char idString[];

void clearLcdRow(int row)
{
  switch(row)
    {
    case 2:
      {
    lcd.locate(0,2);
    lcd.printf("                    ");
    line2Column = 0;
    line2BufIndex = 0;
    break;
      }

    case 3:
      {
    lcd.locate(0,3);
    lcd.printf("                    ");
    line3Column = 0;
    line3BufIndex = 0;
    if (lcdDisplayState != BOTH_ROWS)     displayLcdStringRow(3, idString);
    break;
      }
    }
}

void sendID(char asciiChar)
{
  int i;

  cwBuf->write(CANNED_MSG_START);
  cwBuf->write(' ');
  cwBuf->write('D');
  cwBuf->write('E');
  cwBuf->write(' ');

  for(i=0; i<strlen(idString); i++)
    {
      cwBuf->write(idString[i]);
    }
  cwBuf->write(' ');
  cwBuf->write(CANNED_MSG_END);
  pc.printf(" de %s\r",idString);
}

void sendQRX(char asciiChar)
{
  cwBuf->write(CANNED_MSG_START);
  cwBuf->write(' ');
  cwBuf->write('Q');
  cwBuf->write('R');
  cwBuf->write('X');
  cwBuf->write(' ');

  cwBuf->write(CANNED_MSG_END);
  pc.printf(" QRX\r");
}

void displayState(char asciiChar)
{
  char tempString[20];

  getCompileTime();
  clearLcdRow(2);

  if (rigEna)
    {
      sprintf(tempString, "RIG:ON ");
    }
  else
    {
      sprintf(tempString, "RIG:OFF ");
    }
  displayLcdStringRow(2, tempString);

  if (monEna)
    {
      sprintf(tempString, "MON:ON");
    }
  else
    {
      sprintf(tempString, "MON:OFF");
    }
  displayLcdStringRow(2, tempString);
}



void clearDisplays(char asciiChar)
{
  initConsole();
  displayConsoleID();
  clearLcdRow(2);
  clearLcdRow(3);

  line2Column = 0;
  line2BufIndex = 0;
  line3Column = 0;
  line3BufIndex = 0;
}


void storeSettings(char asciiChar)
{
  fileOutputSettings();
}


void toggleLcdDsplyMode(char asciiChar)
{
  switch(lcdDisplayState)
    {
    case ROW2_ONLY:
      {
    lcdDisplayState = BOTH_ROWS;
    clearLcdRow(2);
    clearLcdRow(3);
    displayLcdStringRow(2,"     BOTH_ROWS      ");
    displayLcdStringRow(3,"     BOTH_ROWS      ");
    break;
      }

    case BOTH_ROWS:
      {
    clearLcdRow(2);
    clearLcdRow(3);
    displayLcdStringRow(3,"     NO_TEXT        ");
    lcdDisplayState = NO_TEXT;
    break;
      }

    case NO_TEXT:
      {
    lcdDisplayState = ROW2_ONLY;
    clearLcdRow(2);
    clearLcdRow(3);
    displayLcdStringRow(3,"     ROW2_ONLY      ");
    break;
      }
    }
}

extern int nrOfRows;
void consoleTextMode(char asciiChar)
{
  switch(nrOfRows)
    {
    case 40: nrOfRows=2; break;
    case 2:  nrOfRows=4; break;
    case 4:  nrOfRows=40; break;
    }
  initConsole();
  displayConsoleID();
}

extern unsigned char allowAutoArrowKeyControl;
void toggleAutoFocus(char asciiChar)
{
  switch(allowAutoArrowKeyControl)
    {
    case  TRUE: allowAutoArrowKeyControl = FALSE; break;
    case FALSE: allowAutoArrowKeyControl = TRUE; break;
    }
}

/** -------------------------------------------------------------------------
@verbatim
This function is called from handleNewKeyPress().  The scancodeToAsciiLookupTable "ascii"
code is passed for possible use by the called function.  The code is a direct index into
the array of handler functions in ctrlFuncs[128].

The scancodeToAsciiLookuptable[4][128] is found in scanCodeToAsciiTables.cpp.  The contents of
this array are gradually being edited for the mbed project.  See additional notes in the comments
where this array is defined.

As an example:  I am adding the "EQUAL" function today (29May2012).  This function is triggered
by the "Insert" key on the keyboard.  The scan code for that key is 0x49.  I went to index hex 49
in the scancodeToAsciiLookuptable and edited the line to contain the custom control code 0xB2:

     0xB2,      // 49* Insert Key (EQUAL)

Prior to this edit, the line had a different code (not 0xB2) and DID NOT have the asterick.  As noted
in the comments for the lookup table:

  // While editing this table to change from the original scan codes used, I will
  // flag edited lines with an * following the index on the line

This can be confusing when coming back into the project "cold" to make a simple function addition!

The next step is to edit ctrlFuncs (below).  The index into this table is generated by subtracting 0x80
from the control code looked up in scancodeToAsciiLookuptable[4][128].  For this example, I subtract 0x80
from 0xB2 to get 0x32.  I have a long "translation table" (it is a comment, not C code) at the end of 
scanCodeToAsciiTables.cpp that I am slowly filling in.  Here is a fragment from that table:

A7 27  dump (keypad 0)
A8 28  msg1 (keypad 1)
A9 29  msg2 (keypad 2)
AA 2A  msg3 (keypad 3)
AB 2B  msg4 (keypad 4)
AC 2C  msg5 (keypad 5)
AD 2D  msg6 (keypad 6)
AE 2E  msg7 (keypad 7)
AF 2F  msg8 (keypad 8)
B0 30  msg9 (keypad 9)
B1 31
B2 32  EQUAL (Insert key)  <--- this is the line of interest in this example
B3 33
B4 34
B5 35  PDL focus
B6 36  KBD focus
B7 37  WT  focus

I use the "32" to guide me to ctrlFunctionNumber32 below, where I then replace "undefinedFunc" with
the new function that will handle this key:

  //ctrlFunctionNumber32, ctrlFunctionNumber31, ctrlFunctionNumber32, ctrlFunctionNumber33,
           sendCannedMsg,        undefinedFunc,  toggleSameSpeedMode,        undefinedFunc,

in this case, the function added is "toggleSameSpeedMode"


@endverbatim
**/
void undefinedFunc(char asciiChar)
{
  pc.printf("Control Function Undefined 0x%02X.\r", asciiChar);
}


void (*ctrlFuncs[128])(char asciiChar) = {

  //ctrlFunctionNumber00, ctrlFunctionNumber01, ctrlFunctionNumber02, ctrlFunctionNumber03,
           undefinedFunc,         dsplyProsign,         dsplyProsign,         dsplyProsign,

  //ctrlFunctionNumber04, ctrlFunctionNumber05, ctrlFunctionNumber06, ctrlFunctionNumber07,
            dsplyProsign,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber08, ctrlFunctionNumber09, ctrlFunctionNumber0A, ctrlFunctionNumber0B,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber0C, ctrlFunctionNumber0D, ctrlFunctionNumber0E, ctrlFunctionNumber0F,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber10, ctrlFunctionNumber11, ctrlFunctionNumber12, ctrlFunctionNumber13,
              swapSpeeds,        undefinedFunc,        clearDisplays,   toggleLcdDsplyMode,

  //ctrlFunctionNumber14, ctrlFunctionNumber15, ctrlFunctionNumber16, ctrlFunctionNumber17,
                 funckey,              funckey,              funckey,              funckey,

  //ctrlFunctionNumber18, ctrlFunctionNumber19, ctrlFunctionNumber1A, ctrlFunctionNumber1B,
                 funckey,              funckey,              funckey,              funckey,

  //ctrlFunctionNumber1C, ctrlFunctionNumber1D, ctrlFunctionNumber1E, ctrlFunctionNumber0F,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber20, ctrlFunctionNumber21, ctrlFunctionNumber22, ctrlFunctionNumber23,
           undefinedFunc,               sendID,        undefinedFunc,         displayState,

  //ctrlFunctionNumber24, ctrlFunctionNumber25, ctrlFunctionNumber26, ctrlFunctionNumber27,
         toggleAutoFocus,        undefinedFunc,        undefinedFunc,           resetCwBuf,

  //ctrlFunctionNumber28, ctrlFunctionNumber29, ctrlFunctionNumber2A, ctrlFunctionNumber2B,
           sendCannedMsg,        sendCannedMsg,        sendCannedMsg,        sendCannedMsg,

  //ctrlFunctionNumber2C, ctrlFunctionNumber2D, ctrlFunctionNumber2E, ctrlFunctionNumber2F,
           sendCannedMsg,        sendCannedMsg,        sendCannedMsg,        sendCannedMsg,

  //ctrlFunctionNumber32, ctrlFunctionNumber31, ctrlFunctionNumber32, ctrlFunctionNumber33,
           sendCannedMsg,        undefinedFunc,  toggleSameSpeedMode,        undefinedFunc,

  //ctrlFunctionNumber34, ctrlFunctionNumber35, ctrlFunctionNumber36, ctrlFunctionNumber37,
           undefinedFunc,             setFocus,             setFocus,             setFocus,

  //ctrlFunctionNumber38, ctrlFunctionNumber39, ctrlFunctionNumber3A, ctrlFunctionNumber3B,
           undefinedFunc,                  up5,                down1,                down5,

  //ctrlFunctionNumber3C, ctrlFunctionNumber3D, ctrlFunctionNumber3E, ctrlFunctionNumber3F,
                     up1,              sendQRX,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber40, ctrlFunctionNumber41, ctrlFunctionNumber42, ctrlFunctionNumber43,
           undefinedFunc,         funckeyShift,         funckeyShift,         funckeyShift,

  //ctrlFunctionNumber44, ctrlFunctionNumber45, ctrlFunctionNumber46, ctrlFunctionNumber47,
            funckeyShift,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber48, ctrlFunctionNumber49, ctrlFunctionNumber4A, ctrlFunctionNumber4B,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber4C, ctrlFunctionNumber4D, ctrlFunctionNumber4E, ctrlFunctionNumber4F,
           undefinedFunc,        undefinedFunc,        undefinedFunc,      consoleTextMode,

  //ctrlFunctionNumber50, ctrlFunctionNumber51, ctrlFunctionNumber52, ctrlFunctionNumber53,
           undefinedFunc,        undefinedFunc,      consoleTextMode,        undefinedFunc,

  //ctrlFunctionNumber54, ctrlFunctionNumber55, ctrlFunctionNumber56, ctrlFunctionNumber57,
            funckeyShift,         funckeyShift,         funckeyShift,         funckeyShift,

  //ctrlFunctionNumber58, ctrlFunctionNumber59, ctrlFunctionNumber5A, ctrlFunctionNumber5B,
            funckeyShift,         funckeyShift,         funckeyShift,         funckeyShift,

  //ctrlFunctionNumber5C, ctrlFunctionNumber5D, ctrlFunctionNumber5E, ctrlFunctionNumber5F,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber60, ctrlFunctionNumber61, ctrlFunctionNumber62, ctrlFunctionNumber63,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        storeSettings,

  //ctrlFunctionNumber64, ctrlFunctionNumber65, ctrlFunctionNumber66, ctrlFunctionNumber67,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber68, ctrlFunctionNumber69, ctrlFunctionNumber6A, ctrlFunctionNumber6B,
              previewMsg,           previewMsg,           previewMsg,           previewMsg,

  //ctrlFunctionNumber6C, ctrlFunctionNumber6D, ctrlFunctionNumber6E, ctrlFunctionNumber6F,
              previewMsg,           previewMsg,           previewMsg,           previewMsg,

  //ctrlFunctionNumber70, ctrlFunctionNumber71, ctrlFunctionNumber72, ctrlFunctionNumber73,
              previewMsg,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber74, ctrlFunctionNumber75, ctrlFunctionNumber76, ctrlFunctionNumber77,
           undefinedFunc,     announcePdlSpeed,     announceKbdSpeed,       announceWeight,

  //ctrlFunctionNumber78, ctrlFunctionNumber79, ctrlFunctionNumber7A, ctrlFunctionNumber7B,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        undefinedFunc,

  //ctrlFunctionNumber7C, ctrlFunctionNumber7D, ctrlFunctionNumber7E, ctrlFunctionNumber7F,
           undefinedFunc,        undefinedFunc,        undefinedFunc,        undefinedFunc,


};
