/** -------------------------------------------------------------------------
Copyright (c) 2012 Chuck Broadwell W5UXH

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

**/

#include "mbed.h"
#include "w5uxhMbedKeyer.h"
#include "queues.h"


//23 JUNE 2012:  COPIED PADDLES.C FROM AVR PROJECT AND MODIFYING IN PLACE TO CONVERT TO MBED

DigitalIn DOT_PDL(p26);
DigitalIn DASH_PDL(p25);

extern unsigned char toggleSameSpeedModeState;


extern unsigned int pdlDotLength_usec;
extern unsigned int pdlDashLength_usec;
extern unsigned int pdlElSpace_usec;  
extern unsigned int pdlCharSpace_usec;
extern unsigned int pdlWordSpace_usec;

extern int cannedMsgState;
unsigned char pdlCwState = IDLE;
unsigned char pdlSpaceState = IDLE;
unsigned char pdlCharSpaceDetected = FALSE;
unsigned char pdlWordSpaceDetected = FALSE;

unsigned char currentElementIsDot  = FALSE;
unsigned char currentElementIsDash = FALSE;
unsigned char dotMemory = FALSE;
unsigned char dashMemory = FALSE;

unsigned char dotAtStart = FALSE;            // set if paddle was closed at start of the interval---
unsigned char dashAtStart = FALSE;           // ---of the element plus element space
unsigned char dotClosedInterval = FALSE;     // set if paddle closed anytime---
unsigned char dashClosedInterval = FALSE;    // ---during the interval
unsigned char dotReleasedInterval = FALSE;   // set if paddle released anytime---
unsigned char dashReleasedInterval = FALSE;  // ---during the interval

unsigned char newPaddleChar;
char currentPaddleChar  = 0x00;
char paddleIdDetectMask = 0x80;

unsigned char allowAutoArrowKeyControl = TRUE;

extern unsigned char cwGenerateMode;
extern unsigned char kbdCwState;
extern Fifo *cwBuf;

extern Timeout cwTimer;

extern unsigned char arrowsControl;
unsigned char curtisType = TYPE_A;

extern Serial pc;

#define NORMAL 0
#define REVERSED 1

#define PDL_IDLE     6
#define PDL_DOT      7
#define PDL_DASH     8
#define PDL_EL_SPACE 9
#define PDL_CHAR_SPACE_DETECT 10
#define PDL_WORD_SPACE_DETECT 11

unsigned char pdlReverseMode = NORMAL;

inline int DOT_PADDLE_CLOSED(void)
{
  if (pdlReverseMode == NORMAL)
    {   
      return(!DOT_PDL);
    }   
  else
    {
      return(!DASH_PDL);
    }
}

inline int DASH_PADDLE_CLOSED(void)
{
  if (pdlReverseMode == NORMAL)
    {   
      return(!DASH_PDL);
    }   
  else
    {
      return(!DOT_PDL);
    }
}


void togglePdlReverseMode(void)
{
  if (pdlReverseMode == NORMAL)
    {
      pdlReverseMode = REVERSED;
      pc.printf("Paddle REVERSED\r");
    }

  else
    {
      pdlReverseMode = NORMAL;
      pc.printf("Paddle NORMAL");
    }

}


void setupPdlDot(void)
{
  keyDown();
  cwTimer.attach_us(&atCwTimer, pdlDotLength_usec);

  pdlSpaceState = IDLE;

  // Now update status items
  pdlCwState = PDL_DOT;
  dotAtStart = TRUE;
  if (DASH_PADDLE_CLOSED())
    {
      dashAtStart = TRUE;
    }
  else
    {
      dashAtStart = FALSE;
    }
  currentElementIsDot = TRUE;
  currentElementIsDash = FALSE;

  // Paddle ID detection operations:
  // For DOT, we only shift the mask.
  paddleIdDetectMask >>= 1;
}

/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/

void setupPdlDash(void)
{
  keyDown();
  cwTimer.attach_us(&atCwTimer, pdlDashLength_usec);

  pdlSpaceState = IDLE;

  // Now update status items
  pdlCwState = PDL_DASH;
  dashAtStart = TRUE;
  if (DOT_PADDLE_CLOSED())
    {
      dotAtStart = TRUE;
    }
  else
    {
      dotAtStart = FALSE;
    }
  currentElementIsDash = TRUE;
  currentElementIsDot  = FALSE;

  // Paddle ID detection operations.
  // The mask has a single logic 1 bit, initially in bit position 7 (MS bit position).
  // For DASH, we OR the mask into the current char, then shift the mask:
  currentPaddleChar |= paddleIdDetectMask;
  paddleIdDetectMask >>= 1;
}


/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/

void setupPdlElementSpace(void)
{
  cwTimer.attach_us(&atCwTimer, pdlElSpace_usec);
  pdlCwState = PDL_EL_SPACE;

  keyUp();
}

/** -------------------------------------------------------------------------
@verbatim
The timer is set for a single element space because the previous element has already
been followed by one element space.  If one more element space duration is detected, then
we are 2/3 of the way to a full character space so we "detect" this as an intended character
space.

@endverbatim
**/

void setupPdlCharSpaceDetect(void)
{
  cwTimer.attach_us(&atCwTimer, pdlCharSpace_usec);
  pdlSpaceState = PDL_CHAR_SPACE_DETECT;
}

/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/

void setupPdlWordSpaceDetect(void)
{
  cwTimer.attach_us(&atCwTimer, pdlWordSpace_usec);
  pdlSpaceState = PDL_WORD_SPACE_DETECT;
}


/** -------------------------------------------------------------------------
@verbatim

During the current element+space interval of either DOT or DASH, monitor the other side of paddle for
state change.  If open at start, look for closure.  If closed at start, look for release.  If
Type=B, store the released paddle for next element.

Set DOT or DASH memory if paddle was open at start of element+space and subsequently closed in the interval.

Use "release" memories for Type B;  set the release memory if corresponding paddle was closed at start and
released in the interval.


@endverbatim
**/

void checkPaddleStatesWhileActive(void)
{
  //For both DOT and DASH paddles, we need to see if they were closed and start and
  //subsequently released, OR
  //If open at start, were they subsequently closed.
  if (dotAtStart)
    {
      if (!DOT_PADDLE_CLOSED())  dotReleasedInterval = TRUE;
    }
  else
    {
      if (DOT_PADDLE_CLOSED()) dotClosedInterval = TRUE;
    }


  if (dashAtStart)
    {
      if (!DASH_PADDLE_CLOSED()) dashReleasedInterval = TRUE;
    }
  else
    {
      if (DASH_PADDLE_CLOSED()) dashClosedInterval = TRUE;
    }
}


/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void resetMemoryFlags(void)
{
  dotMemory   = FALSE;
  dashMemory  = FALSE;
}

/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/

void resetStartAndIntervalFlags(void)
{
  dotAtStart  = FALSE;
  dashAtStart = FALSE;

  dotClosedInterval    = FALSE;
  dotReleasedInterval  = FALSE;

  dashClosedInterval   = FALSE;
  dashReleasedInterval = FALSE;
}

/** -------------------------------------------------------------------------
@verbatim


@endverbatim
**/
void checkPaddles(void)
{
  if ((DASH_PADDLE_CLOSED()) && (pdlCwState == IDLE))
    {
      if (cwBuf->get_count() != 0)
    {
      cwBuf->reset();
      wait_ms(250);
      cannedMsgState = IDLE;
      return;
    }

      if (cwGenerateMode != PDL)
    {
      cwGenerateMode = PDL;
    }

      checkAutoFocus(PDL);
      resetStartAndIntervalFlags();
      resetMemoryFlags();
      setupPdlDash();
    }

  if ((DOT_PADDLE_CLOSED()) && (pdlCwState == IDLE))
    {
      if (cwBuf->get_count() != 0)
    {
      cwBuf->reset();
      wait_ms(250);
      cannedMsgState = IDLE;
      return;
    }

      if (cwGenerateMode != PDL)
    {
      cwGenerateMode = PDL;
    }

      checkAutoFocus(PDL);
      resetStartAndIntervalFlags();
      resetMemoryFlags();
      setupPdlDot();
    }


  if (pdlCwState != IDLE)
    {
      checkPaddleStatesWhileActive();
    }
}


/** -------------------------------------------------------------------------
@verbatim 

15 Aug 2010:  First success, seem to have Iambic and basic Type A and Type B working.
              

@endverbatim
**/

void doIambic(void)
{
  unsigned char nextElement;

  nextElement = PDL_IDLE;

  // If dot was closed during the interval, set the dot memory
  if (dotClosedInterval) dotMemory = TRUE;

  // If dash was closed during interval, set the dash memory
  if (dashClosedInterval) dashMemory = TRUE;

  if (curtisType == TYPE_B)
    {
      if (currentElementIsDash)
    {
      if ((dotAtStart) && (dotReleasedInterval)) dotMemory = TRUE;
    }
      if (currentElementIsDot)
    {
      if ((dashAtStart) && (dashReleasedInterval)) dashMemory = TRUE;
    }
    }

  resetStartAndIntervalFlags();

  if ((currentElementIsDash) && (dotMemory))  nextElement = PDL_DOT;
  if ((currentElementIsDot)  && (dashMemory)) nextElement = PDL_DASH;

  if (nextElement == PDL_IDLE)
    {
      if ((DOT_PADDLE_CLOSED()) && (DASH_PADDLE_CLOSED()))
    {
      if (currentElementIsDash) nextElement = PDL_DOT;
      if (currentElementIsDot)  nextElement = PDL_DASH;
    }
      else
    {
      if (DASH_PADDLE_CLOSED()) nextElement = PDL_DASH;
      if (DOT_PADDLE_CLOSED())  nextElement = PDL_DOT;
    }
    }


  switch (nextElement)
    {
    case PDL_DASH: 
      {
    resetStartAndIntervalFlags();
    resetMemoryFlags();
    setupPdlDash(); 
    break;
      }

    case PDL_DOT: 
      {
    resetStartAndIntervalFlags();
    resetMemoryFlags();
    setupPdlDot(); 
    break;
      }

    default:
      {
    pdlCwState = IDLE;

    kbdCwState = KBD_IDLE;

    // Set up to test if a character space exists:
    setupPdlCharSpaceDetect();
      }
    }
}



/** -------------------------------------------------------------------------
@verbatim
This is the specific handler for Timer1 when PDL is active.

@endverbatim
**/
// atCwTimer() calls this handler if PDL is active
void atCwTimerPdl(void)
{
  switch(pdlCwState)
    {
    case PDL_DOT: case PDL_DASH:
      {
    // End of an element, so start element space
    setupPdlElementSpace();
    break;
      }
    case PDL_EL_SPACE:
      {
    // We have just completed an element space that followed either a DOT or a DASH
    // Thus we now need to check the various paddle flags to see if any paddles either
    // released or closed during the element+space interval.
    doIambic();
    break;
      }
    }


  switch(pdlSpaceState)
    {
    case PDL_CHAR_SPACE_DETECT:
      {
    // If we get here, then we have a potential character space (which was assume is correct timing!)
    // Do the "final" OR operation for the terminating bit.
    // Set flag for main idle loop to pick up.
    currentPaddleChar |= paddleIdDetectMask;
    pdlSpaceState = IDLE;
    pdlCharSpaceDetected = TRUE;
    newPaddleChar = currentPaddleChar;

    // reinitialize for next character
    currentPaddleChar = 0x00;
    paddleIdDetectMask = 0x80;

    // then setup to detect word space
    setupPdlWordSpaceDetect();
    break;
      }

    case PDL_WORD_SPACE_DETECT:
      {
    pdlSpaceState = IDLE;
    pdlWordSpaceDetected = TRUE;
    break;
      }
    }
}


/** -------------------------------------------------------------------------
@verbatim

This section will duplicate fragments of the PIC code, with comments
added to attept to "translate" from PIC to "pseudo code" and then use
this as a reference for the actual C code in this module.

In the code fragments below, sections are deleted when not pertinent
to decoding the algorithm.

---------------------------------------------------------------------
IN MAIN IDLE LOOP WHILE NO PADDLE ELEMENTS IN PROCESS

WE CHECK TO SEE IF ANY PADDLE ACTIONS ARE CURRENTLY IN PROGRESS;  IF
NOT WE THEN CHECK FOR PADDLE CLOSURES:


init_1
    btfss    cw_state,idlestate ;if not idlestate, paddles may be active
    goto    not_idle2
TEST DOT PADDLE FOR CLOSURE:
                ;else check paddles if is idlestate
    btfsc    PORTC,dot_paddle    ;see if dot paddle closed
    goto    dot_pdl_not_closed

DOT PADDLE FOUND TO BE CLOSED, SO PREPARE TO INITIATE A DOT:
setup_dot1    
    btfss    PORTC,dot_paddle
    bsf    bf2,dot_at_start
NO IDEA WHY THE DOT PADDLE IS TESTED A SECOND TIME (ABOVE) IMMEDIATELY AFTER
HAVING JUST BEEN TESTED AND FOUND TO BE CLOSED!

NEXT, WE TEST TO SEE IF THE DASH IS *ALSO* CLOSED AND IF SO WE STORE THE FACT:
    btfss    PORTC,dash_paddle
    bsf    bf2,dash_at_start

THEN WE SET THE FLAG TO SHOW THAT DOT IS THE CURRENT ("LAST") ELEMENT:
    bsf    bf3,last_el_was_dot

AND WE CLEAR THE FLAG TO SHOW THAT THE CURRENT ELEMENT IS NOT DASH:
    bcf    bf3,last_el_was_dash

AND WE *CLEAR* THE DOT MEMORY
    bcf    bf3,dotmem

SET THE ACTIVE FLAG, CLEAR THE DASH STATE FLAG, SET THE DOT STATE FLAG:
    bsf    bf1,active
    bcf    bf1,dash_state    ; make sure this is cleared
    bsf    bf1,dot_state


    bcf    bf1,space_idle
    bcf    cw_state,idlestate
    goto    start_dot

TEST DASH PADDLE FOR CLOSURE:
dot_pdl_not_closed    
    btfsc    PORTC,dash_paddle    ;see if dash paddle closed
    goto    dash_pdl_not_closed

DASH PADDLE FOUND TO BE CLOSED, SO PREPARE TO INITIATE A DASH:
setup_dash1    
    btfss    PORTC,dot_paddle
    bsf    bf2,dot_at_start
    btfss    PORTC,dash_paddle
    bsf    bf2,dash_at_start
ONCE AGAIN, THE "CURRENT" PADDLE IS TESTED ALONG WITH THE "OPPOSITE", IT JUST DOES NOT SEEM AS
IF IT SHOULD BE NECESSARY TO TEST THE DASH IN THIS CASE

THEN WE SET THE FLAG TO SHOW THAT DASH IS THE CURRENT ("LAST") ELEMENT:
    bsf    bf3,last_el_was_dash

AND WE CLEAR THE FLAG TO SHOW THAT THE CURRENT ELEMENT IS NOT DOT:
    bcf    bf3,last_el_was_dot

AND WE CLEAR THE DASH MEMORY
    bcf    bf3,dashmem

SET THE ACTIVE FLAG, CLEAR THE DOT STATE FLAG, SET THE DASH STATE FLAG:
    bsf    bf1,active    ;paddles active
    bcf    bf1,dot_state    ; make sure this is cleared
    bsf    bf1,dash_state

    bcf    bf1,space_idle
    bcf    cw_state,idlestate

    goto    start_dash


THIS MAY JUST BE AN EXTRA CHECK OF PADDLES, NOT SURE HOW IT FITS IN
not_idle2
    btfss    bf1,active    ;see if paddles active
    goto    paddles_not_active
    btfss    bf1,space_idle
    goto    not_idle3
            ;if space_idle, also check for new closures
    btfss    PORTC,dash_paddle
    goto    setup_dash
    btfss    PORTC,dot_paddle
    goto    setup_dot
    goto    not_idle3


---------------------------------------------------
IN MAIN IDLE LOOP WHILE CW IS ACTIVE, NEED TO MONITOR PADDLE STATE CHANGES

its_not_done            ;look for changes in paddle states
                ;want to set dot_released_int flag
                ;if dot was closed at start and has
                ;since been released


    btfss    bf2,dot_at_start  ;if dot was closed at start, need to see
                ;if released, so skip next if was closed
                ;at start
    goto    ov2        ;jump, since was not closed at start
                ;

IF DOT WAS CLOSED AT START, WE CHECK TO SEE IF IT HAS BEEN RELEASED
WE SET THE dot_at_start FLAG IF DOT IS CLOSED AT START OF EITHER DOT OR DASH
                ;it was closed at start, see if now released
    btfsc    PORTC,dot_paddle ;skip next if not released, clear=not rlsd

IF DOT PADDLE IS NOT CLOSED, WE SET THE FLAG TO STORE THIS FACT:
    bsf    bf2,dot_released_int ;else was released in the interval
    goto    ov3

ov2            ;dot was not closed at start, so see if closed now
            ;want dot_closed_int flag set if dot was open
            ;at start and has been closed in the interval
    btfss    PORTC,dot_paddle    ;skip next if not closed, set=not

WE ALSO SET THE "CLOSED" FLAG IF DOT WAS NOT CLOSED AT START BUT HAS NOW BEEN CLOSED
    bsf    bf2,dot_closed_int    ;else was closed in the interval

ov3            ;we have now handled the dot paddle history
            ;where dot was closed at start, so handle dash
            ;in similiar way

SAME STEPS FOR DASH, IF IT WAS CLOSED AT START WE CHECK TO SEE IF IT HAS BEEN RELEASED, OTHERWISE
WE CHECK TO SEE IF IT HAS BEEN CLOSED.
    btfss    bf2,dash_at_start
    goto    ov4
    btfsc    PORTC,dash_paddle
    bsf    bf2,dash_released_int
    goto    ov5
ov4
    btfss    PORTC,dash_paddle
    bsf    bf2,dash_closed_int
ov5

    goto    loop


---------------------------------------------------
AT END OF CURRENT ELEMENT+SPACE; NEED TO DETERMINE NEXT ELEMENT IF ANY

iambic
    btfsc    bf2,dot_closed_int
    bsf    bf3,dotmem
IF DOT WAS CLOSED DURING THE INTERVAL, WE SET THE DOT MEMORY (ABOVE)

    btfsc    bf2,dash_closed_int
    bsf    bf3,dashmem
IF DASH WAS CLOSED DURING THE INTERVAL, WE SET THE DASH MEMORY (ABOVE)

    btfss    bf3,type
    goto    not_type_B




BELOW, WE HANDLE THE EXTRA STEPS IF CURTIS MODE IS B:

FOR TYPE B, WE NEED TO DETECT IF THE OPPOSITE PADDLE HAD BEEN CLOSED AT THE START OF THE CURRENT
ELEMENT (THAT HAS JUST COMPLETED), AND IF THE OPPOSITE PADDLE HAD BEEN CLOSED, AND WAS SUBSEQUENTLY 
RELEASED DURING THE ELEMENT+SPACE INTERVAL, WE SET THE MEMORY FOR THAT SIDE OF PADDLE SO THAT
A TRAILING FINAL ELEMENT WILL BE GENERATED.

    btfsc    bf3,last_el_was_dot
    goto    ia1            ;if finishing dot, don't check
                    ;for dot release

IF CURRENT ELEMENT IS *NOT* DOT, SEE IF A RELEASE OF DOT PADDLE WAS DETECTED:
    btfsc    bf2,dot_released_int


WE ONLY GET HERE IF THE CURRENT ELEMENT WAS A DASH AND IF THE DOT PADDLE HAD BEEN CLOSED AT 
THE START AND SUBSEQUENTLY RELEASED;  WE THUS SET THE DOT MEMORY
    bsf    bf3,dotmem


ia1
    btfsc    bf3,last_el_was_dash
    goto    ia2

IF THE CURRENT ELEMENT WAS A DOT, AND IF DASH HAD BEEN CLOSED AT START, WE CHECK TO SEE IF
IT WAS SUBSEQUENTLY RELEASED, AND IF SO WE SET THE DASH MEMORY.
    btfsc    bf2,dash_released_int
    bsf    bf3,dashmem





AND THEN CONTINUE WITH STEPS COMMON TO BOTH B AND A:
ia2
not_type_B

HERE WE CLEAR ALL OF THE FLAGS IN bf2:

bf2        equ    50h    ;bit flag set number 2
                                ;bit flag 2 bits   CAUTION:  CLRF USED ON THIS BYTE, SO ALL BITS RELATED
dot_at_start                    equ    0    ;set if paddle was closed at start
dash_at_start                    equ    1    ;..of interval of element plus space
dot_closed_int                    equ    2    ;set if paddle closed anytime during the
dash_closed_int                    equ    3    ; interval (int)
dot_released_int                equ    4    ;set if paddle released anytime during
dash_released_int               equ    5    ; the interval

    clrf    bf2

WE TEST TO SEE IF THE LAST ELEMENT WAS A DASH, IN WHICH CASE THE btfsc INSTRUCTION *DOES NOT* SKIP
IT SKIPS IF THE FLAG WAS CLEAR, SO IF IT WAS SET, MEANING THE LAST ELEMENT WAS A DASH, WE GO TO
CHECK TO SEE IF THE DOT MEMORY IS SET, IN WHICH CASE WE THEN GO SET UP A DOT
    btfsc    bf3,last_el_was_dash
    goto    see_if_need_dot


LIKEWISE, IF LAST ELEMENT WAS A DOT, WE GO TO CHECK THE DASH MEMORY AND START A DASH IF IT IS SET
    btfsc    bf3,last_el_was_dot
    goto    see_if_need_dash
    goto    s2

see_if_need_dot
    btfsc    bf3,dotmem
    goto    setup_dot
    goto    s1

see_if_need_dash
    btfsc    bf3,dashmem
    goto    setup_dash


IF NEITHER MEMORY WAS SET, WE TEST THE PADDLES ONE MORE TIME FOR POSSIBLE NEW CLOSURES.
s2
    btfss    PORTC,dash_paddle
    goto    setup_dash
s1
    btfss    PORTC,dot_paddle
    goto    setup_dot


@endverbatim
**/
void A01_AlgorithmFromPIC_Code(void)
{
  // dummy function strictly for Doxygen
}



/** -------------------------------------------------------------------------
@verbatim

  // dummy function strictly for Doxygen

These are hand transcribed notes taken from a rough sheet of Algorithm
notes in the old PIC keyer notebook.  It is unknown how closely the
PIC firmware followed these notes.

These notes would have been generated some time in the 1990's when
working on the PIC paddle keyers.

Starting from IDLE state:

1)
When dot or dash paddle closure detected, switch to active element
state for dot or dash.  During the element and subsequent element
space, look for closure of opposite paddle if it was open at start of
element; if closure detected, store this fact.  Otherwise look for
release (does not indicate if release only of the opposite paddle
though) and if Type B, store as next element (so it surely means only
the opposite paddle).  There is no mention of monitoring the state of
the paddle corresponding with the current element.

2)
Only set dot or dash memory if corresponding paddle was open at start
of element+space and subsequently closed in that interval.

3)
Use "release" memories for Type B.  Only set the release memory if
corresponding paddle was closed at start of element+space and released
during the interal.

--------------------------------------------------------------
At end of element space:

1)  Type A

If last element was DASH and dot memory is set, start dot, else if
dash paddle is closed, start dash.

If last element was DOT and dash memory is set, start dash, else if 
dot paddle is closed, start dot.

If no memories are set and both paddles are closed, new element will
be opposite of previous element

2) Type B

All as above, plus:

If "release" memory is set, new element = which ever release memory is set.


@endverbatim
**/
void A00_AlgorithmText(void)
{
  // dummy function strictly for Doxygen
}
