

/** -------------------------------------------------------------------------
@verbatim
For initial testing, this array is a place holder evolved from the AVR project, so the contents
are not that functionally useful.

The column of data provides either the ASCII byte or my special function codes.
The order of entries is in scan code order (e.g. scancode 00, then 01 etc.) 

The [4] index represents four of the possible modifier bytes:  

[0] = normal (lower case,no modifier)
[1] = CTRL   (LEFT control only)
[2] = SHIFT  (either LEFT or RIGHT shift)
[3] = ALT    (LEFT alt only)

The RIGHT CTRL and ALT keys will be decoded alone as special functions.

@endverbatim


**/



unsigned char scancodeToAsciiLookupTable[4][128] = 
{
  // First 128 entries are "normal" (no shift or ctrl or alt)
  // While editing this table to change from the original scan codes used, I will
  // flag edited lines with an * following the index on the line
  // I am easily confused when I come back to add new edits.  Keep in mind that if the
  // line does not have an *, then the only thing currently "up to date" on the line is
  // the sequential index hex count starting from 00 and going up to 7F.
  // OOPS:  The * was used in the unshifted portion of the table, not necessarily
  //        for the other sections!!!!!  So lack of the * may not indicate an entry is not updated.
  // When the lookup from this table is one of the codes >= 0x80, it eventually is then
  // used as the index into the ctrlFuncs[128] control function table found in ctrlCommands.cpp.
  // The ctrlFuncs[] table stores the addresses of functions that are invoked to handle special
  // control functions for the most part, but a few are for the prosigns on function keys F1-F4.
          0,    // 00
          0,    // 01
          0,    // 02
          0,    // 03
        'a',    // 04* a
        'b',    // 05* b
        'c',    // 06* c
        'd',    // 07* d
        'e',    // 08* e
        'f',    // 09* f
        'g',    // 0A* g
        'h',    // 0B* h
        'i',    // 0C* i
        'j',    // 0D* j
        'k',    // 0E* k
        'l',    // 0F* l (ell)
        'm',    // 10* m
        'n',    // 11* n
        'o',    // 12* o
        'p',    // 13* p
        'q',    // 14* q
        'r',    // 15* r
        's',    // 16* s
        't',    // 17* t
        'u',    // 18* u
        'v',    // 19* v
        'w',    // 1A* w
        'x',    // 1B* x
        'y',    // 1C* y
        'z',    // 1D* z
        '1',    // 1E* 1
        '2',    // 1F* 2
        '3',    // 20* 3
        '4',    // 21* 4
        '5',    // 22* 5
        '6',    // 23* 6
        '7',    // 24* 7
        '8',    // 25* 8
        '9',    // 26* 9
        '0',    // 27* 0
       0x0D,    // 28* BT (ENTER key)
       0x9C,    // 29*  ESC
          8,    // 2A* Backspace
          9,    // 2B* Tab (AA)
        ' ',    // 2C* space
        '-',    // 2D* -  (dash)
        '=',    // 2E* =
        '[',    // 2F* [
        ']',    // 30* ]
        '/',    // 31* backspace, but using fwd slash for now to avoid "escape" 
        'm',    // 32  m
        ';',    // 33* ;
         39,    // 34* ' (single quote)
       0x9D,    // 35* ~
        ',',    // 36* , (comma)
        '.',    // 37* . (period)
        '/',    // 38* /
       0xA1,    // 39* ID  (caps lock key) 
     0x81,      // 3A* F1
     0x82,      // 3B* F2
     0x83,      // 3C* F3
     0x84,      // 3D* F4
     0x94,      // 3E* F5
     0x95,      // 3F* F6
     0x96,      // 40* F7
     0x97,      // 41* F8
     0x98,      // 42* F9
     0x99,      // 43* F10
     0x9A,      // 44* F11
     0x9B,      // 45* F12
     0xA3,      // 46  
     0xA3,      // 47* scroll lock
     0xAF,      // 48  keypad 8
     0xB2,      // 49* Insert Key (EQUAL)
     0xA6,      // 4A  keypad -
     0xAB,      // 4B  keypad 4
     0xB5,      // 4C* Delete
     0xB6,      // 4D* End
     0xB7,      // 4E* PgDn
     0xBC,      // 4F* right arrow (UP   1)
     0xBA,      // 50* left  arrow (DOWN 1)
     0xBB,      // 51* down  arrow (DOWN 5)
     0xB9,      // 52* up    arrow (UP   5)
     0xA4,      // 53* NumLock (toggle auto ctrl)
        0,      // 54  
        0,      // 55  
        0,      // 56
     0x9A,      // 57  F11
     0xBD,      // 58* keypad ENTER (QRX)
     0xA8,      // 59* keypad 1
     0xA9,      // 5A* keypad 2  
     0xAA,      // 5B* keypad 3
     0xAB,      // 5C* keypad 4
     0xAC,      // 5D* keypad 5
     0xAD,      // 5E* keypad 6
     0xAE,      // 5F* keypad 7
     0xAF,      // 60* keypad 8
     0xB0,      // 61* keypad 9
     0xA7,      // 62* keypad 0
        0,      // 63  prtscr
     0x90,      // 64  Right Alt
     0x92,      // 65* (APPS) clear display
     0xB3,      // 66  home
     0xB9,      // 67  Up Arrow
     0xB4,      // 68  page up
     0xBA,      // 69  Left Arrow
     0xBC,      // 6A  Right Arrow
     0xB6,      // 6B  end (KBD)
     0xBB,      // 6C  Down Arrow
     0xB7,      // 6D  pgdn (WT)
     0xB2,      // 6E
     0xB5,      // 6F  delete (PDL)
        0,      // 70  
        0,      // 71  
        0,      // 72  
        0,      // 73  
        0,      // 74  
        0,      // 75  
        0,      // 76  
        0,      // 77  break
        0,      // 78
        0,      // 79
        0,      // 7A
        0,      // 7B
        0,      // 7C
        0,      // 7D
     0x91,      // 7E
     0x92,      // 7F

        0,      // 00  ****CTRL SCANCODES START HERE****
     0xDC,      // 01  ESC
        '!',    // 02  1
        '@',    // 03  2
        '#',    // 04  3
        '$',    // 05  4
        '%',    // 06  5
        '^',    // 07  6
        '&',    // 08  7
        '*',    // 09  8
        '(',    // 0A  9
        ')',    // 0B  0
        '_',    // 0C  dash
        '+',    // 0D  =
     0x08,      // 0E  backspace
     0x09,      // 0F  fwd tab
        'Q',    // 10  q
        'W',    // 11  w
        'E',    // 12  e
        'R',    // 13  r
        'T',    // 14  t
        'Y',    // 15  y
        'U',    // 16  u
        'I',    // 17  i
        'O',    // 18  o
        'P',    // 19  p
        '{',    // 1A  [
        '}',    // 1B  ]
     0x0D,      // 1C  Enter (main keys)
     0xA1,      // 1D  Left Ctrl
        'A',    // 1E  a
        'S',    // 1F  s
        'D',    // 20  d
        'F',    // 21  f
        'G',    // 22  g
        'H',    // 23  h
        'J',    // 24  j
        'K',    // 25  k
        'L',    // 26  l  (ell)
        ':',    // 27  ;
         34,    // 28  '  (double quote = decimal 34)
          0,    // 29  nil
          0,    // 2A  Left Shift
        '|',    // 2B  \  (backslash = decimal 92)
        'Z',    // 2C  z
        'X',    // 2D  x
        'C',    // 2E  c
        'V',    // 2F  v
        'B',    // 30  b
        'N',    // 31  n
        'M',    // 32  m
        '<',    // 33  , (comma)
        '>',    // 34  . period
        '?',    // 35  /
        0,      // 36  Right Shift
        0,      // 37
     0x9F,      // 38  Left Alt
        ' ',    // 39  Space
     0x0D,      // 3A  Caps Lock
     0xC1,      // 3B  F1
     0xC2,      // 3C  F2
     0xC3,      // 3D  F3
     0xC4,      // 3E  F4
     0xC4,      // 3F  F5
     0xC5,      // 40  F6
     0xC6,      // 41  F7
     0xC7,      // 42  F8
     0xC8,      // 43  F9
     0xC9,      // 44  F10
     0xA4,      // 45  numlock
     0xE3,      // 46  scroll lock
     0xEE,      // 47  keypad 7
     0xEF,      // 48  keypad 8
     0xE0,      // 49  keypad 9
     0xE6,      // 4A  keypad -
     0xEB,      // 4B  keypad 4
     0xEC,      // 4C  keypad 5
     0xED,      // 4D  keypad 6
     0xE1,      // 4E  keypad +
     0xE8,      // 4F  keypad 1
     0xE9,      // 50  keypad 2
     0xEA,      // 51  keypad 3
     0xAB,      // 52  keypad 0
     0xB8,      // 53  keypad .
        0,      // 54  
        0,      // 55  
        0,      // 56
     0xCA,      // 57  F11
     0xCB,      // 58  F12
        0,      // 59  
        0,      // 5A  
        0,      // 5B  
        0,      // 5C
        0,      // 5D  
        0,      // 5E
        0,      // 5F
     0xBD,      // 60  keypad Enter
        0,      // 61
     0xA0,      // 62  keypad /
        0,      // 63  prtscr
     0x90,      // 64  Right Alt
     0xCF,      // 65* ctrl-APPS key, consoleTextMode
     0xB3,      // 66  home
     0xB9,      // 67  Up Arrow
     0xB4,      // 68  page up
     0xBA,      // 69  Left Arrow
     0xBC,      // 6A  Right Arrow
        0,      // 6B  
     0xBB,      // 6C  Down Arrow
        0,      // 6D
        0,      // 6E
        0,      // 6F
        0,      // 70  
        0,      // 71  
        0,      // 72  
        0,      // 73  
        0,      // 74  
        0,      // 75  
        0,      // 76  
        0,      // 77  break
        0,      // 78
        0,      // 79
        0,      // 7A
        0,      // 7B
        0,      // 7C
        0,      // 7D
     0x91,      // 7E
     0x92,       // 7F

        0,      // 00  ****SHIFT SCANCODES START HERE****
     0x9C,      // 01  ESC
        '1',    // 02  1
        '2',    // 03  2
        '3',    // 04  3
        '4',    // 05  4
        '5',    // 06  5
        '6',    // 07  6
        '7',    // 08  7
        '8',    // 09  8
        '9',    // 0A  9
        '0',    // 0B  0
        '-',    // 0C  dash
        '=',    // 0D  =
     0x08,      // 0E  backspace
     0x09,      // 0F  fwd tab
        'q',    // 10  q
        'w',    // 11  w
        'e',    // 12  e
        'r',    // 13  r
        't',    // 14  t
        'y',    // 15  y
        'u',    // 16  u
        'i',    // 17  i
        'o',    // 18  o
        'p',    // 19  p
        '[',    // 1A  [
        ']',    // 1B  ]
     0x0D,      // 1C  Enter (main keys)
     0xA1,      // 1D  Left Ctrl
        '!',    // 1E* !
        '@',    // 1F* @
        '#',    // 20* #
        '$',    // 21* $
        '%',    // 22* %
        '^',    // 23* ^
        '&',    // 24* &
        '*',    // 25* *
        '(',    // 26* (
        ')',    // 27* )
       0x0D,    // 28* enter
          0,    // 29* nil
          8,    // 2A* backspace
          9,    // 2B* tab
        ' ',    // 2C* space
        '_',    // 2D* underscore
        '+',    // 2E* +
        '{',    // 2F* {
        '}',    // 30* }
        '|',    // 31* |
        'm',    // 32
        ':',    // 33* :
        '"',    // 34* "
        '~',    // 35* ~
       0x83,    // 36* AR
       0x84,    // 37* SK
        '?',    // 38* ?
     0x0D  ,    // 39* BT (caps lock key)  (need change to ID)
     0xC1,      // 3A* F1 shifted
     0xC2,      // 3B* F2
     0xC3,      // 3C* F3
     0xC4,      // 3D* F4
     0xD4,      // 3E* F5
     0xD5,      // 3F* F6
     0xD6,      // 40* F7
     0xD7,      // 41* F8
     0xD8,      // 42* F9
     0xD9,      // 43* F10
     0xDA,      // 44* F11
     0xDB,      // 45* F12 shifted
     0xA3,      // 46  
     0xE3,      // 47* scroll lock
     0xAF,      // 48  keypad 8
     0x90,      // 49* insert
     0xA6,      // 4A  keypad -
     0xAB,      // 4B  keypad 4
     0xF5,      // 4C* Delete   announce pdl speed
     0xF6,      // 4D* End      announce kbd speed
     0xF7,      // 4E* PgDn     announce weight
     0xA8,      // 4F  keypad 1
     0xA9,      // 50  keypad 2
     0xAA,      // 51  keypad 3
     0xA7,      // 52  keypad 0
     0xB8,      // 53  keypad .
        0,      // 54  
        0,      // 55  
        0,      // 56
     0x9A,      // 57  F11
     0x9B,      // 58  F12
     0xE8,      // 59* keypad 1  
     0xE9,      // 5A* keypad 2  
     0xEA,      // 5B* keypad 3  
     0xEB,      // 5C* keypad 4  
     0xEC,      // 5D* keypad 5  
     0xED,      // 5E* keypad 6  
     0xEE,      // 5F* keypad 7  
     0xEF,      // 60* keypad 8  
     0xF0,      // 61* keypad 9  
     0x91,      // 62  keypad /
        0,      // 63  prtscr
     0x90,      // 64  Right Alt
     0x93,      // 65* shift-APPS cycle display mode
     0xB3,      // 66  home
     0xB9,      // 67  Up Arrow
     0xB4,      // 68  page up
     0xBA,      // 69  Left Arrow
     0xBC,      // 6A  Right Arrow
     0xB6,      // 6B  end (KBD)
     0xBB,      // 6C  Down Arrow
     0xB7,      // 6D  pgdn (WT)
     0xB2,      // 6E
     0xB5,      // 6F  delete (PDL)
        0,      // 70  
        0,      // 71  
        0,      // 72  
        0,      // 73  
        0,      // 74  
        0,      // 75  
        0,      // 76  
        0,      // 77  break
        0,      // 78
        0,      // 79
        0,      // 7A
        0,      // 7B
        0,      // 7C
        0,      // 7D
     0x91,      // 7E
     0x92,      // 7F

        0,      // 00  ****ALT SCANCODES START HERE****
     0xDC,      // 01  ESC
        '!',    // 02  1
        '@',    // 03  2
        '#',    // 04  3
        '$',    // 05  4
        '%',    // 06  5
        '^',    // 07  6
        '&',    // 08  7
        '*',    // 09  8
        '(',    // 0A  9
        ')',    // 0B  0
        '_',    // 0C  dash
        '+',    // 0D  =
     0x08,      // 0E  backspace
     0x09,      // 0F  fwd tab
        'Q',    // 10  q
        'W',    // 11  w
        'E',    // 12  e
        'R',    // 13  r
        'T',    // 14  t
        'Y',    // 15  y
        'U',    // 16  u
        'I',    // 17  i
        'O',    // 18  o
        'P',    // 19  p
        '{',    // 1A  [
        '}',    // 1B  ]
     0x0D,      // 1C  Enter (main keys)
     0xA1,      // 1D  Left Ctrl
        'A',    // 1E  a
        'S',    // 1F  s
        'D',    // 20  d
        'F',    // 21  f
        'G',    // 22  g
        'H',    // 23  h
        'J',    // 24  j
        'K',    // 25  k
        'L',    // 26  l  (ell)
        ':',    // 27  ;
         34,    // 28  '  (double quote = decimal 34)
          0,    // 29  nil
          0,    // 2A  Left Shift
        '|',    // 2B  \  (backslash = decimal 92)
        'Z',    // 2C  z
        'X',    // 2D  x
        'C',    // 2E  c
        'V',    // 2F  v
        'B',    // 30  b
        'N',    // 31  n
        'M',    // 32  m
        '<',    // 33  , (comma)
        '>',    // 34  . period
        '?',    // 35  /
        0,      // 36  Right Shift
        0,      // 37
     0x9F,      // 38  Left Alt
        ' ',    // 39  Space
     0x0D,      // 3A  Caps Lock
     0xC1,      // 3B  F1
     0xC2,      // 3C  F2
     0xC3,      // 3D  F3
     0xC4,      // 3E  F4
     0xC4,      // 3F  F5
     0xC5,      // 40  F6
     0xC6,      // 41  F7
     0xC7,      // 42  F8
     0xC8,      // 43  F9
     0xC9,      // 44  F10
     0xA4,      // 45  numlock
     0xE3,      // 46  scroll lock
     0xEE,      // 47  keypad 7
     0xEF,      // 48  keypad 8
     0xE0,      // 49  keypad 9
     0xE6,      // 4A  keypad -
     0xEB,      // 4B  keypad 4
     0xEC,      // 4C  keypad 5
     0xED,      // 4D  keypad 6
     0xE1,      // 4E  keypad +
     0xE8,      // 4F  keypad 1
     0xE9,      // 50  keypad 2
     0xEA,      // 51  keypad 3
     0xAB,      // 52  keypad 0
     0xB8,      // 53  keypad .
        0,      // 54  
        0,      // 55  
        0,      // 56
     0xCA,      // 57  F11
     0xCB,      // 58  F12
        0,      // 59  
        0,      // 5A  
        0,      // 5B  
        0,      // 5C
        0,      // 5D  
        0,      // 5E
        0,      // 5F
     0xBD,      // 60  keypad Enter
        0,      // 61
     0xA0,      // 62  keypad /
        0,      // 63  prtscr
     0x90,      // 64  Right Alt
        0,      // 65
     0xB3,      // 66  home
     0xB9,      // 67  Up Arrow
     0xB4,      // 68  page up
     0xBA,      // 69  Left Arrow
     0xBC,      // 6A  Right Arrow
        0,      // 6B  
     0xBB,      // 6C  Down Arrow
        0,      // 6D
        0,      // 6E
        0,      // 6F
        0,      // 70  
        0,      // 71  
        0,      // 72  
        0,      // 73  
        0,      // 74  
        0,      // 75  
        0,      // 76  
        0,      // 77  break
        0,      // 78
        0,      // 79
        0,      // 7A
        0,      // 7B
        0,      // 7C
        0,      // 7D
     0x91,      // 7E
     0x92       // 7F

  };




#if 0
Custom Control Codes:  We subtract 0x80 from the codes to do the lookup function into the command function
                       table.

80 00  This code is not used for a key;  it is the flag code for end of message in buffer.
81 01  F1 Not a command, it is the AS prosign.
82 02  F2 Not a command, it is the VE prosign.
83 03  F3 Not a command, it is the AR prosign.
84 04  F4 Not a command, it is the SK prosign.
85 05  
86 06  
87 07
88 08
89 09
8A 0A
8B 0B
8C 0C
8D 0D
8E 0E
8F 0F
90 10 SWAP SPEEDS (shift-Insert key)
91 11
92 12 shift-APPS toggle lcd dsply mode
93 13 APPS key   clear dsply
94 14 F5
95 15 F6
96 16 F7
97 17 F8
98 18 F9
99 19 F10
9A 1A F11
9B 1B F12
9C 1C
9D 1D
9E 1E
9F 1F
A0 20
A1 21 ID (caps lock)
A2 22
A3 23 scroll lock (display timestamp etc.)
A4 24 num lock (toggle auto focus)
A5 25
A6 26
A7 27  dump (keypad 0)
A8 28  msg1 (keypad 1)
A9 29  msg2 (keypad 2)
AA 2A  msg3 (keypad 3)
AB 2B  msg4 (keypad 4)
AC 2C  msg5 (keypad 5)
AD 2D  msg6 (keypad 6)
AE 2E  msg7 (keypad 7)
AF 2F  msg8 (keypad 8)
B0 30  msg9 (keypad 9)
B1 31
B2 32  EQUAL (Insert key)
B3 33
B4 34
B5 35  PDL focus
B6 36  KBD focus
B7 37  WT  focus
B8 38
B9 39  up5
BA 3A  down1
BB 3B  down5
BC 3C  up1
BD 3D  QRX
BE 3E
BF 3F
C0 40
C1 41 F1 shifted
C2 42 F2 shifted
C3 43 F3 shifted
C4 44 F4 shifted
C5 45
C6 46
C7 47
C8 48
C9 49
CA 4A
CB 4B
CC 4C
CD 4D
CE 4E
CF 4F ctrl-APPS  consoleTextMode
D0 50
D1 51
D2 52 
D3 53
D4 54 F5 shifted
D5 55 F6 shifted
D6 56 F7 shifted
D7 57 F8 shifted
D8 58 F9 shifted
D9 59 F10 shifted
DA 5A F11 shifted
DB 5B F12 shifted
DC 5C
DD 5D
DE 5E
DF 5F
E0 60
E1 61
E2 62
E3 63  scroll lock shifted (store settings to file)
E4 64
E5 65
E6 66
E7 67
E8 68  shift-keypad1
E9 69  shift-keypad2
EA 6A  shift-keypad3
EB 6B  shift-keypad4
EC 6C  shift-keypad5
ED 6D  shift-keypad6
EE 6E  shift-keypad7
EF 6F  shift-keypad8
F0 70  shift-keypad9
F1 71
F2 72
F3 73
F4 74
F5 75  PDL announce pdl speed
F6 76  KBD announce kbd speed
F7 77  WT  announce weight
F8 78
F9 79
FA 7A
FB 7B
FC 7C
FD 7D
FE 7E
FF 7F


#endif


#if 0
These are from the AVR project.  There will certainly be changes in the mbed project.

UNSHIFTED CODES:

0x80 0x81 0x82 0x83 0x84 0x85 0x86 0x87 0x88 0x89 0x8A 0x8B 0x8C 0x8D 0x8E 0x8F
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
see    F1   F2   F3   F4
note   AS   VE   AR   SK
above  
     

0x90 0x91 0x92 0x93 0x94 0x95 0x96 0x97 0x98 0x99 0x9A 0x9B 0x9C 0x9D 0x9E 0x9F
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
RITE RGUI APPS RITE  F5   F6   F7   F8   F9   F10  F11  F12  ESC   ~       LGUI 
 ALT           CTRL
SWAP TOG  CLR       TMR            MON  RIG1           TUNE      BOTH
SPD  DSPL 2&3       RST            ENA   ENA

0xA0 0xA1 0xA2 0xA3 0xA4 0xA5 0xA6 0xA7 0xA8 0xA9 0xAA 0xAB 0xAC 0xAD 0xAE 0xAF
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
KPAD LEFT LEFT SCR  NUM  KPAD KPAD KPAD KPAD KPAD KPAD KPAD KPAD KPAD KPAD KPAD
SLSH CTRL  ALT LOCK LOCK   *    -    0    1    2    3    4    5    6    7    8
               STOR               DUMP MSG1 MSG2 MSG3 MSG4 MSG5 MSG6 MSG7 MSG8
               SPDS

0xB0 0xB1 0xB2 0xB3 0xB4 0xB5 0xB6 0xB7 0xB8 0xB9 0xBA 0xBB 0xBC 0xBD 0xBE 0xBF
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
KPAD KPAD      KPAD KPAD KPAD KPAD KPAD KPAD  UP  LEFT  DWN RITE KPAD      KPAD
  9    +       HOME PGUP  DEL  END PGDN   .  ARRW ARRW ARRW ARRW ENTR      INS
MSG9                      PDL  KBD  WT       SP+5 SP-1 SP-5 SP+1         


SHIFTED CODES:

0xC0 0xC1 0xC2 0xC3 0xC4 0xC5 0xC6 0xC7 0xC8 0xC9 0xCA 0xCB 0xCC 0xCD 0xCE 0xCF
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
see   F1   F2   F3   F4
note  
above  


0xD0 0xD1 0xD2 0xD3 0xD4 0xD5 0xD6 0xD7 0xD8 0xD9 0xDA 0xDB 0xDC 0xDD 0xDE 0xDF
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
RITE RGUI APPS RITE  F5   F6   F7   F8   F9  F10  F11  F12 ESC     ~       LGUI 
 ALT           CTRL

0xE0 0xE1 0xE2 0xE3 0xE4 0xE5 0xE6 0xE7 0xE8 0xE9 0xEA 0xEB 0xEC 0xED 0xEE 0xEF
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
KPAD LEFT LEFT CTRL NUM  KPAD KPAD KPAD KPAD KPAD KPAD KPAD KPAD KPAD KPAD KPAD
SLSH CTRL  ALT APPS LOCK   *    -    0    1    2    3    4    5    6    7    8
                                  DUMP MSG1 MSG2 MSG3 MSG4 MSG5 MSG6 MSG7 MSG8
                                       -----------REPEAT MESSAGES-------------


0xF0 0xF1 0xF2 0xF3 0xF4 0xF5 0xF6 0xF7 0xF8 0xF9 0xFA 0xFB 0xFC 0xFD 0xFE 0xFF
---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- 
KPAD KPAD      KPAD KPAD KPAD KPAD KPAD KPAD  UP  LEFT  DWN RITE KPAD      KPAD
  9    +       HOME PGUP  DEL  END PGDN   .  ARRW ARRW ARRW ARRW ENTR      INS
MSG9                                                               
RPT                                                               

NOTE:  I am using shifted code 0xE3 for the ctrl-APPS key to control the LCD display
mode for the bottom two lines (similar to the AVR right-CTRL key function)

#endif
